@@ -5,17 +5,13 @@
 #include "jellyfish.h"
 
 #define NOTNUM(c)   ((c>57) || (c<48))
-#define INRANGE(c)  ((c>0)  && (c<91))
-
-#ifndef NaN
-#define NaN (0.0 / 0.0)
-#endif
 
 /* borrowed heavily from strcmp95.c
  *    http://www.census.gov/geo/msb/stand/strcmp.c
  */
-double _jaro_winkler(const char *ying, const char *yang,
-                     bool long_tolerance, bool winklerize)
+double _jaro_winkler(const JFISH_UNICODE *ying, int ying_length,
+                     const JFISH_UNICODE *yang, int yang_length,
+                     int long_tolerance, int winklerize)
 {
     /* Arguments:
 
@@ -30,11 +26,11 @@ double _jaro_winkler(const char *ying, const char *yang,
          test when comparing fixed length fields such as phone and
          social security numbers.
     */
-    char *ying_flag=0, *yang_flag=0;
+    JFISH_UNICODE *ying_flag=0, *yang_flag=0;
 
     double weight;
 
-    long ying_length, yang_length, min_len;
+    long min_len;
     long search_range;
     long lowlim, hilim;
     long trans_count, common_chars;
@@ -42,21 +38,21 @@ double _jaro_winkler(const char *ying, const char *yang,
     int i, j, k;
 
     // ensure that neither string is blank
-    ying_length = strlen(ying);
-    yang_length = strlen(yang);
     if (!ying_length || !yang_length) return 0;
 
     search_range = min_len = (ying_length > yang_length) ? ying_length : yang_length;
 
     // Blank out the flags
-    ying_flag = alloca(ying_length + 1);
-    if (!ying_flag) return NaN;
+    ying_flag = calloc((ying_length + 1), sizeof(JFISH_UNICODE));
+    if (!ying_flag) {
+        return -100;
+    }
 
-    yang_flag = alloca(yang_length + 1);
-    if (!yang_flag) return NaN;
-
-    memset(ying_flag, 0, ying_length + 1);
-    memset(yang_flag, 0, yang_length + 1);
+    yang_flag = calloc((yang_length + 1), sizeof(JFISH_UNICODE));
+    if (!yang_flag) {
+        free(ying_flag);
+        return -100;
+    }
 
     search_range = (search_range/2) - 1;
     if (search_range < 0) search_range = 0;
@@ -78,7 +74,11 @@ double _jaro_winkler(const char *ying, const char *yang,
     }
 
     // If no characters in common - return
-    if (!common_chars) return 0;
+    if (!common_chars) {
+        free(ying_flag);
+        free(yang_flag);
+        return 0;
+    }
 
     // Count the number of transpositions
     k = trans_count = 0;
@@ -126,16 +126,20 @@ double _jaro_winkler(const char *ying, const char *yang,
         }
     }
 
+    free(ying_flag);
+    free(yang_flag);
     return weight;
 }
 
 
-double jaro_winkler(const char *ying, const char *yang, bool long_tolerance)
+double jaro_winkler_distance(const JFISH_UNICODE *ying, int ying_len,
+        const JFISH_UNICODE *yang, int yang_len,
+        int long_tolerance)
 {
-    return _jaro_winkler(ying, yang, long_tolerance, true);
+    return _jaro_winkler(ying, ying_len, yang, yang_len, long_tolerance, 1);
 }
 
-double jaro_distance(const char *ying, const char *yang)
+double jaro_distance(const JFISH_UNICODE *ying, int ying_len, const JFISH_UNICODE *yang, int yang_len)
 {
-    return _jaro_winkler(ying, yang, false, false);
+    return _jaro_winkler(ying, ying_len, yang, yang_len, 0, 0);
 }
